c----------------------------------------------------------------------
c --- CALSUM  -- Sum CALPUFF concentration or flux output files
c----------------------------------------------------------------------
c
c --- CALSUM     Version: 1.5.0     Level: 121203               MAIN
c
c     Copyright (c) 2014 by Exponent, Inc.
c
c --- Written by:  J. Scire
c                  EARTH TECH / Sigma Research
c
c
c **********************************************************************
c --- Exponent, Inc. Updates:
c **********************************************************************
c --- Version 1.4, Level: 110301 to Version 1.5.0, Level: 121203
c      -  Allow files with different time periods to be merged,
c         assuming that times that are missing from a CALPUFF
c         file have ZERO impact from the sources contained in the file
c      -  Expand version string to 12 characters
c      New     :  TIMEHEAD
c      Modified:  (main), RDWR1, RDWRHR
c      Included:  CALUTILS (remove inline UTCBASR)
c **********************************************************************
c
c --- Version 1.33, L051122 to Version 1.4, L110301
c     (1) Add DATASET 2.2 CALPUFF output files
c      New     :  UTCBASR (from CALUTILS)
c      Modified:  RDWR1
c
c --- Version 1.32, L040726 to Version 1.33, L051122
c     (1) Change language to replace "hour" with "period" because
c         CALPUFF output does not need to be hourly
c      Modified:  main
c
c --- Version 1.31, L040623 to Version 1.32, L040726
c     (1) Dimension QA arrays as MXFIL+MXSRC because each file with
c         source contributions contains a TOTALS entry in addition to
c         entries for each source.  Arrays also are started from 1
c         instead of 0.
c      Modified:  RDWRHR
c
c     (2) Compute the source number within each type as data records
c         are processed from the scratch files.  For example, file 1
c         may contain point sources 1 and 2, and file 2 may contain
c         point sources 1-3.  The output must identify these as point
c         sources 1,2,3,4,5.
c      Modified:  RDWRHR
c
c --- Version 1.3, L031017 to Version 1.31, L040623
c     (1) Use binary OPEN for scratch files associated with DATASET
c         2.1 CALPUFF output files
c      Modified:  RDWRHR
c
c --- Version 1.21, L030709 to Version 1.3, L031017
c     (1) Changes to read DATASET 2.1 CALPUFF output files
c      Modified:  PARAMS.SUM, RDWR1, RDWRHR
c
c --- Version 1.2, L030402 to Version 1.21, L030709
c     (1) Fixed counter for number of hours processed
c     (2) Add subroutine CLIPR to drop any characters after the 
c         first string of non-blank characters when extracting
c         filenames from control file
c
c --- Version 1.0, L021028 to Version 1.2, L030402
c --- Modified by F. Robe, D. Strimaitis
c     ** Requires DATASET 2.0 CALPUFF output format **
c     (1) Changes to read NOOBS CALPUFF files (FRR)
c      -  Additional parameter I2DRH (flag for 2D arrays of rel.hum)
c     (2) Changes to read/write extended header records (DGS)
c      -  All control file documentation from first file retained
c         in header of output file;  only title information from other
c         files retained in output file
c      -  /MAP/ variables added
c
c --- Modified by J. Scire 2/99
c       (1) Allows pollutant-specific scaling factors
c       (2) New option to allow compressed or uncompressed summed
c           output files
c
c --- Modified by F. Robe - Earth Tech (Ver. 1.0, Level: 961219)
c                    12/96 - Modified to read in a combination of 
c                            compressed and uncompressed input files
c                            (variable LCOMPR changed to array LCOMPR
c                            (MXFIL). Still uncompressed output file 
c                            only.
c                    10/96 - Modified to read in compressed CALPUFF 
c                            files, i.e. choice of either all 
c                            compressed or all uncompressed input 
c                            CALPUFF files.  Uncompressed output only
c
c----------------------------------------------------------------------
c
c --- Include parameters
      include 'params.sum'
c
      real ascale(mxfil,mxspec),bscale(mxfil,mxspec)
      integer nstot(8),nsrc(mxfil)

c --- v1.5.0 (121203)
      integer nstep(2,mxfil), kuse(mxfil)

      character*80 titleo(3)
      character*70 cfile(mxfil),cfilout

c --- v1.5.0 (121203)
      character*8 clevel
      character*12 cver

      logical lsamp,lend
      logical lcompr(mxfil),lcompro
c
c --- Set version and level numbers
c --- v1.5.0 (121203)
      cver='1.5.0'
      clevel='121203'

c --- Open control & list files
      open(io5,file='calsum.inp',status='old')
      open(io6,file='calsum.lst',status='unknown')
c
c --- Write the version/level numbers
      write(io6,2)cver,clevel

c --- v1.5.0 (121203)
2     format(1x,'CALSUM -- Version: ',a12,3x,'Level: ',a8)

c
c ----------------------------
c --- Read control file inputs
c ----------------------------
c
c --- Read the number of files to sum (NFILES)
      read(io5,*)nfiles
      if(nfiles.gt.mxfil)then
      write(io6,*)'ERROR - too many files for array dimensions -- ',
     1   'NFILES = ',nfiles,' MXFIL = ',mxfil
      stop
      endif
c
c --- Read input file names (CFILE(mxfil))
      do i=1,nfiles
         read(io5,'(a70)')cfile(i)
         call CLIPR(cfile(i))
      enddo
c
c --- Read output file name (CFILOUT)
      read(io5,'(a70)')cfilout
      call CLIPR(cfilout)
c
c --- Read the compression option flag for the OUTPUT file
      read(io5,*)lcompro
c
c --- Read # species & scaling factors for each file ("nfiles" lines)
c --- (ASCALE is multiplicative factor, BSCALE is additive factor)
      read(io5,*)nspec
      if(nspec.gt.mxspec)then
         write(io6,*)
         write(io6,*)
         write(io6,*)
         write(io6,*)'ERROR -- NSPEC is too large for array dimension'
         write(io6,*)'NSPEC = ',nspec,' MXSPEC = ',mxspec
         stop 'CALSUM Halted --- See List File'
      endif
c
      do i=1,nfiles
         read(io5,*)(ascale(i,n),bscale(i,n),n=1,nspec)
      enddo
c --- Read the 3-line title of the new, summed file (TITLEO(3))
      do i=1,3
         read(io5,'(a80)')titleo(i)
      enddo
c
c -----------------------------------------
c --- Echo back the inputs to the list file
c -----------------------------------------
      write(io6,*)'NFILES = ',nfiles
      write(io6,*)
      write(io6,*)'Names of input files: '
      write(io6,*)
      do i=1,nfiles
         write(io6,10)i,cfile(i)
10       format(5x,'No. ',i3,3x,'Filename: ',a70)
      enddo
      write(io6,*)
      write(io6,*)'Name of output file: '
      write(io6,12)cfilout
12    format(/8x,'Output Filename: ',a70)
c
      write(io6,*)
      write(io6,*)'Output file compressed? (LCOMPRO) = ',lcompro
c --- Write scaling factors for each file
      write(io6,14)nspec
14    format(//1x,'Scaling factors (of form X(new) = X(old)*ASCALE + ',
     1 'BSCALE) for each species'//1x,'No. species (NSPEC) = ',i6)
      do i=1,nfiles
         write(io6,16)i,(ascale(i,n),bscale(i,n),n=1,nspec)
16       format(1x,'File: ',i5,'(ASCALE, BSCALE): '/1p,
     1   (5x,5(e12.5,6x,e12.5)/5x,5(e12.5,6x,e12.5)/))
      enddo
      write(io6,17)
17    format(/1x,'BSCALE is in grams/m**3'//1x,'NOTE: When ',
     1 'using BSCALE to add a constant background concentration '/
     2 7x,'this is normally done by apply BSCALE to one CALPUFF '/
     3 7x,'output file only, with SCALE=0 for the other files')
      write(io6,*)
      write(io6,*)'Title lines on new output file:'
      do i=1,3
         write(io6,18)titleo(i)
18       format(5x,a80)
      enddo

c --- Transfer control file image to scratch file
      call TFERCF(titleo,ncommout)

c
c --- Open the input files
      do i=1,nfiles
         io=io21+i-1
         open(io,file=cfile(i),status='old',form='unformatted')
      enddo
c
c --- Open the output file
      open(io10,file=cfilout,status='unknown',form='unformatted')
c
c --- Read and write the header records
c frr 02/10/28: i2DRH flag (NOOBS)

c --- v1.5.0 (121203)
      call rdwr1(cver,clevel,ncommout,titleo,nfiles,lcompro,nspeci,
     1           nreci,nctreci,lsamp,nxsam,nysam,iflux,lcompr,i2drh,
     2           iptime,isrcinfo,isrcindv,ntsrc,nsrc,nstot,nstep)
c --- Set the total number of periods to be processed
      mxnstep=0
      do k=1,nfiles
         mxnstep=MAX(mxnstep,nstep(2,k))
      enddo

c
c --- Check that number of species match value in control file
      if(nspec.ne.nspeci)then
         write(io6,*)'ERROR -- no. species in file does not ',
     1   'match value specified in the control file'
         stop 'CALSUM Halted --- See List File'
      endif
c
c --- Loop over all periods in the file
      nhours=0

c --- v1.5.0 (121203)
      do kstep=1,mxnstep
c --- Identify which files have data to read/write this step
      do k=1,nfiles
         if(nstep(1,k).LE.kstep .AND. nstep(2,k).GE.kstep) then
            kuse(k)=1
         else
            kuse(k)=0
         endif
      enddo
      call rdwrhr(nfiles,nspeci,nreci,nctreci,lsamp,nxsam,nysam,
     1            iflux,ascale,bscale,lcompr,lcompro,
     2            iptime,isrcinfo,isrcindv,ntsrc,nsrc,nstot,kuse,
     3            lend)
      if(lend) then
c ---    Potential problem: EOF is not expected
         goto 999
      else
         nhours=nhours+1
      endif
      enddo
999   continue
      write(io6,*)
      write(io6,*)'Number of periods expected : ',mxnstep
      write(io6,*)'Number of periods processed: ',nhours

      stop
      end

c----------------------------------------------------------------------
c --- Get subroutines from CALUTILS
      include 'calutils.for'
c----------------------------------------------------------------------


c----------------------------------------------------------------------
c --- v1.5.0 (121203)
      subroutine rdwr1(cver,clev,ncommout,titleo,nfiles,lcompro,
     1  nspeci,nreci,nctreci,lsamp,nxsam,nysam,iflux,lcompr,i2drh,
     2  iptime_,isrcinfo_,isrcindv_,ntsrc,nsrc,nstot,nstep)
c----------------------------------------------------------------------
c
c --- CALSUM     Version: 1.5.0     Level: 121203              RDWR1
c                J. Scire
c
c --- PURPOSE:  Read & write the header records of the CALPUFF
c               output files (concentration, dry flux, and wet flux)
c               
c --- UPDATE
c --- V1.5.0 (121203) from V1.4 (110301)
c                    - Allow time periods to be different
c                    - Version string is a12
c --- V1.4 (110301) from V1.3 (031017) (DGS)
c                    - CALPUFF.DAT dataset version 2.2 format
c                    - All files must be same dataset version
c --- V1.3 (031017) from V1.2 (030402) (DGS)
c                    - CALPUFF.DAT dataset version 2.1 format
c --- V1.2 (030402) from V1.0 (990210) (DGS)
c                    - CALPUFF control file documentation from first
c                      file retained in header of output file
c                    - CALPUFF.DAT dataset version 2.0 format
c---- Modifications: - FRR(4/02) add 2D RH array flag
c
c --- INPUTS:
c --- v1.5.0 (121203)
c                 CVER - Char.*12    - CALSUM version
c                 CLEV - Char.*8     - CALSUM level
c             NCOMMOUT - integer     - Number of comment records saved
c                                      for output header
c            TITLEO(3) - Char.*80    - New run title (3 lines of
c                                      80 char. each) that will be
c                                      output to the new, summed file
c               NFILES - integer     - Number of files to be summed
c              LCOMPRO - logical     - Compression option for output
c                                      file: F=uncompressed, 
c                                            T=compressed
c       Parameters:
c             MXNZ, MXREC, MXRECT, MXSPEC, MXFIL, MXSRC
c             IO6, IO10, IO21
c
c --- OUTPUT:
c               NSPECI - integer     - Number of species in files
c                NRECI - integer     - Number of discrete receptors
c              NCTRECI - integer     - Number of special complex terrain
c                LSAMP - logical     - Flag indicating if gridded
c                                      receptors used (T=yes, F=no)
c                                      (CTSG) receptors
c                NXSAM - integer     - No. cells in X direction of
c                                      gridded receptor grid
c                NYSAM - integer     - No. cells in Y direction of
c                                      gridded receptor grid
c                IFLUX - integer     - Flag indicating if file are
c                                      concentrations (IFLUX=0) or
c                                      wet/dry fluxes (IFLUX=1).  The
c                                      CTSG receptors have no flux data.
c        LCOMPR(mxfil) - logical arr -Compression flags for each input 
c                                      file
c                i2DRH - integer     - flag for 2D RH arrays
c              IPTIME_ - integer     - Date-time format flag
c            ISRCINFO_ - integer     - Source information flag
c            ISRCINDV_ - integer     - Individual source contribution
c                                      flag
c                NTSRC - integer     - Total number of sources
c          NSRC(mxfil) - integer arr - Number of sources in each file
c             NSTOT(8) - integer arr - Number of sources of each type
c --- v1.5.0 (121203)
c       NSTEP(2,mxfil) - integer arr - Step number to the first data in
c                                      each file, and to the last data
c                                      data in each file
c
c
c --- RDWR1 called by:  MAIN
c --- v1.5.0 (121203)
c --- RDWR1 calls:      COMPRS, TIMEHEAD
c
c----------------------------------------------------------------------
c
c --- Include parameters
      include 'params.sum'
c
      real dxkm(mxfil),dykm(mxfil),xorigkm(mxfil),yorigkm(mxfil)
      real xrec(mxrec,mxfil),yrec(mxrec,mxfil),zrec(mxrec,mxfil)
      real xrct(mxrect,mxfil),yrct(mxrect,mxfil),zrct(mxrect,mxfil)

      integer ibyr(mxfil),ibjul(mxfil),ibhr(mxfil),irlg(mxfil),
     1 iavg(mxfil),nx(mxfil),ny(mxfil),ione(mxfil),nssta(mxfil),
     2 ibcomp(mxfil),iecomp(mxfil),jbcomp(mxfil),jecomp(mxfil),
     3 ibsamp(mxfil),jbsamp(mxfil),iesamp(mxfil),jesamp(mxfil),
     4 meshdn(mxfil),npts(mxfil),nareas(mxfil),nlines(mxfil),
     5 nvols(mxfil),nrec(mxfil),nctrec(mxfil),nspout(mxfil),
     6 i2dmet(mxfil)

c --- v1.5.0 (121203)
      integer nstep(2,mxfil)

c --- Variables added for DATASET VERSION 2.2
c     ASPUNIT(mxspec,mxfil) units name for each species
c     ABTZ(mxfil)           UTC+hhmm time zone string

c --- Variables added for DATASET VERSION 2.1
c --- Notes:  
c     NSTYPE(mxfil,8) holds the number of each type of source
c                     in each file (npt1, npt2, nar1, etc.)
c     NSTOT(8)        holds the total number of each type of source
c                     from all files (npt1, npt2, nar1, etc.)
      real xbtz(mxfil)
      integer ibsec(mxfil),nsecdt(mxfil),nstype(mxfil,8),nstot(8),
     &        msource(mxfil),nsrc(mxfil)
      integer iptime(mxfil),isrcinfo(mxfil),isrcindv(mxfil)
      character*16 csource(mxsrc,8)
      character*4 cstype(8)

      integer ihill(mxrect,mxfil)
      logical lcompr(mxfil),lcompro, lfatal
      logical lspunit

      character*8 abtz(mxfil)

c --- v1.5.0 (121203)
      character*12 cver
      character*8 clev

      character*80 titleo(3)
      character*80 title(3,mxfil)
      character*15 csout(mxspec,mxfil)
      character*12 cmodel(mxfil),ver(mxfil),level(mxfil)
      character*16 aspunit(mxspec,mxfil)

      logical lsgrid(mxfil),lsamp

c --- Arrays for mapping parameters
      character*4 utmhem(mxfil)
      character*8 datum(mxfil),pmap(mxfil)
      character*12 daten(mxfil)
      character*16 clat0(mxfil),clon0(mxfil),clat1(mxfil),clat2(mxfil)
      integer iutmzn(mxfil)
      real feast(mxfil),fnorth(mxfil)
      real rnlat0(mxfil),relon0(mxfil),xlat1(mxfil),xlat2(mxfil)

c --- Local Variables for header text
      character*16 dataset,dataver,dataset0,dataver0
      character*33 blank33,break33
      character*64 datamod
      character*132 doc1,comment1,blank,break

      data blank33/'                                 '/
      data break33/'.................................'/

c --- Configure output documentation
      data doc1/'Produced by CALSUM Version: '/

c --- Set names of each source type for error report section
      data cstype/'pt1 ','pt2 ','ar1 ','ar2 ',
     1            'ln1 ','ln2 ','vl1 ','vl2 '/

      lspunit=.FALSE.

c --- Set blank (132 characters)
      blank(1:33)=blank33
      blank(34:66)=blank33
      blank(67:99)=blank33
      blank(100:132)=blank33

c --- Set break (132 characters)
      break(1:33)=break33
      break(34:66)=break33
      break(67:99)=break33
      break(100:132)=break33

c --- Initialize total number of sources
      ntsrc=0
      do k=1,8
         nstot(k)=0
      enddo

c --- Construct the version-level comment string
      j=29

c --- v1.5.0 (121203)
      do i=1,12

         if(cver(i:i).NE.' ') then
            doc1(j:j)=cver(i:i)
            j=j+1
         endif
      enddo
      j=j+1
      comment1(j:j+7)=' Level: '
      j=j+8
      do i=1,8
         if(clev(i:i).NE.' ') then
            doc1(j:j)=clev(i:i)
            j=j+1
         endif
      enddo
c
c ----------------------------------------------------------------------
c --- INPUT header record data from input files
c ----------------------------------------------------------------------
c
c --- Read all of the information from "NFILES" CALPUFF files
      do i=1,nfiles
         io=io21+i-1

c ---    Read and test first record to check header format
c ---    Record #1 - File Declaration -- 24 words
         read(io) dataset,dataver,datamod
         ifilver=0
         if(dataset.EQ.'CONC.DAT') then
            ifilver=1
         elseif(dataset.EQ.'DFLX.DAT') then
            ifilver=1
         elseif(dataset.EQ.'WFLX.DAT') then
            ifilver=1
         endif

         if(ifilver.EQ.0) then
c ---       File format is not supported
            write(*,*)'Header records indicate unsupported format'
            write(*,*)'for CALPUFF data file!  FILE = ',i
            write(*,*)'Expected datset version 2.0 or later'
            stop
         endif
c
c ---    Store dataset name
         dataset0=dataset

         if(i.EQ.1) then
c ---       Save dataset version
            dataver0=dataver
c ---       Retain all comment records for output file
c ---       To scratch file
            write(iox,'(a132)') blank
            write(iox,'(a132)') break
            write(iox,'(a132)') blank
            write(iox,'(2a16,a64)') dataset,dataver,datamod
            ncommout=ncommout+4
c ---       Number of comment records
            read(io) ncom
            ncommout=ncommout+ncom
            do k=1,ncom
               comment1=blank
               read(io) comment1
               write(iox,'(a132)') comment1
            enddo
         else
c ---       Keep only CALPUFF control file title information
            keep=1
            kout=0
c ---       To scratch file
            write(iox,'(a132)') blank
            write(iox,'(a132)') break
            write(iox,'(a132)') blank
            write(iox,'(2a16,a64)') dataset,dataver,datamod
            ncommout=ncommout+4
c ---       Number of comment records
            read(io) ncom
            do k=1,ncom
               comment1=blank
               read(io) comment1
               dataset=comment1(1:16)
               if(keep.EQ.1) then
c ---             Pass records until another dataset is found
                  if(dataset.EQ.'CALMET.DAT') then
                     keep=0
                     kout=0
                  elseif(dataset.EQ.dataset0) then
                     keep=1
                     kout=0
c ---                To scratch file
                     write(iox,'(a132)') comment1
                     ncommout=ncommout+1
                  else
                     kout=kout+1
                     if(kout.LE.4) then
c ---                   To scratch file
                        write(iox,'(a132)') comment1
                        ncommout=ncommout+1
                     endif
                  endif
               elseif(keep.EQ.0) then
c ---             Skip records until CALPUFF dataset is found
                  if(dataset.EQ.dataset0) then
                     keep=1
                     kout=0
c ---                To scratch file
                     write(iox,'(a132)') comment1
                     ncommout=ncommout+1
                  endif
               endif
            enddo
         endif
c
c ---    HEADER RECORD NCOM+3 -- General run parameters
c
         if(dataver.EQ.'2.0             ') then
           read(io)cmodel(i),ver(i),level(i),ibyr(i),ibjul(i),ibhr(i),
     1       irlg(i),iavg(i),nx(i),ny(i),dxkm(i),dykm(i),ione(i),
     2       xorigkm(i),yorigkm(i),nssta(i),ibcomp(i),iecomp(i),
     3       jbcomp(i),jecomp(i),ibsamp(i),jbsamp(i),iesamp(i),
     4       jesamp(i),meshdn(i),npts(i),nareas(i),nlines(i),nvols(i),
     5       nrec(i),nctrec(i),lsgrid(i),nspout(i),lcompr(i),i2dmet(i),
     6       iutmzn(i),feast(i),fnorth(i),rnlat0(i),relon0(i),
     7       xlat1(i),xlat2(i),pmap(i),utmhem(i),datum(i),daten(i),
     8       clat0(i),clon0(i),clat1(i),clat2(i)
           msource(i)=0
           iptime(i)=1
           isrcinfo(i)=0
           isrcindv(i)=0
           nsrc(i)=0

         elseif(dataver.EQ.'2.1             ') then
           read(io)cmodel(i),ver(i),level(i),ibyr(i),ibjul(i),ibhr(i),
     1       ibsec(i),xbtz(i),irlg(i),iavg(i),nsecdt(i),nx(i),ny(i),
     2       dxkm(i),dykm(i),ione(i),xorigkm(i),yorigkm(i),nssta(i),
     3       ibcomp(i),iecomp(i),jbcomp(i),jecomp(i),ibsamp(i),
     4       jbsamp(i),iesamp(i),jesamp(i),meshdn(i),nstype(i,1),
     5       nstype(i,2),nstype(i,3),nstype(i,4),nstype(i,5),
     6       nstype(i,6),nstype(i,7),nstype(i,8),msource(i),
     7       nrec(i),nctrec(i),lsgrid(i),nspout(i),lcompr(i),i2dmet(i),
     8       iutmzn(i),feast(i),fnorth(i),rnlat0(i),relon0(i),
     9       xlat1(i),xlat2(i),pmap(i),utmhem(i),datum(i),daten(i),
     x       clat0(i),clon0(i),clat1(i),clat2(i)
           iptime(i)=2
           isrcinfo(i)=1
           isrcindv(i)=msource(i)
           npts(i)=nstype(i,1)+nstype(i,2)
           nareas(i)=nstype(i,3)+nstype(i,4)
           nlines(i)=nstype(i,5)+nstype(i,6)
           nvols(i)=nstype(i,7)+nstype(i,8)
           nsrc(i)=npts(i)+nareas(i)+nlines(i)+nvols(i)
           ntsrc=ntsrc+nsrc(i)

         elseif(dataver.EQ.'2.2             ') then
           read(io)cmodel(i),ver(i),level(i),ibyr(i),ibjul(i),ibhr(i),
     1       ibsec(i),abtz(i),irlg(i),iavg(i),nsecdt(i),nx(i),ny(i),
     2       dxkm(i),dykm(i),ione(i),xorigkm(i),yorigkm(i),nssta(i),
     3       ibcomp(i),iecomp(i),jbcomp(i),jecomp(i),ibsamp(i),
     4       jbsamp(i),iesamp(i),jesamp(i),meshdn(i),nstype(i,1),
     5       nstype(i,2),nstype(i,3),nstype(i,4),nstype(i,5),
     6       nstype(i,6),nstype(i,7),nstype(i,8),msource(i),
     7       nrec(i),nctrec(i),lsgrid(i),nspout(i),lcompr(i),i2dmet(i),
     8       iutmzn(i),feast(i),fnorth(i),rnlat0(i),relon0(i),
     9       xlat1(i),xlat2(i),pmap(i),utmhem(i),datum(i),daten(i),
     x       clat0(i),clon0(i),clat1(i),clat2(i)
           iptime(i)=2
           isrcinfo(i)=1
           isrcindv(i)=msource(i)
           npts(i)=nstype(i,1)+nstype(i,2)
           nareas(i)=nstype(i,3)+nstype(i,4)
           nlines(i)=nstype(i,5)+nstype(i,6)
           nvols(i)=nstype(i,7)+nstype(i,8)
           nsrc(i)=npts(i)+nareas(i)+nlines(i)+nvols(i)
           ntsrc=ntsrc+nsrc(i)
c ---     Convert from UTC string to real base time zone
          call UTCBASR(abtz(i),xbtz(i))
c ---     Specie-level units are in header
          lspunit=.TRUE.

         else
           write(io6,*)
           write(io6,*)
           write(io6,*)
           write(io6,*)'ERROR:  Unknown DATASET Version: ',dataver
           stop 'Halted in RDWR1 --- See List File'
         endif
c
c ----------------------------------------------------
c ---    Perform QA checks related to array dimensions
c ----------------------------------------------------
c
c ---    Check dataset version
         if(dataver.NE.dataver0)then
            write(io6,*)
            write(io6,*)
            write(io6,*)
            write(io6,*)'ERROR -- Datset Version is different ',
     1         ' -- File: ',i
            write(io6,*)'Expected match with file 1: '//dataver0
            write(io6,*)'     Found dataset version: '//dataver
            stop 'Halted in RDWR1 --- See List File'
         endif
c
c ---    Check number of species
         if(nspout(i).gt.mxspec)then
            write(io6,*)
            write(io6,*)
            write(io6,*)
            write(io6,*)'ERROR -- No. species too large for array ',
     1         'dimensions -- File: ',i
            write(io6,*)'NSPOUT(i) = ',nspout(i),' MXSPEC = ',mxspec
            stop 'Halted in RDWR1 --- See List File'
         endif
c
c ---    Check number of gridded receptors
         ngridx=meshdn(i)*(iesamp(i)-ibsamp(i))+1
         ngridy=meshdn(i)*(jesamp(i)-jbsamp(i))+1
         if(ngridx.gt.mxnxg.or.ngridy.gt.mxnyg)then
            write(io6,*)
            write(io6,*)
            write(io6,*)
            write(io6,*)'ERROR -- Too many gridded receptors for ',
     1         'array dimensions -- File: ',i
            write(io6,*)'NGRIDX = ',ngridx,' MXNXG = ',mxnxg
            write(io6,*)'IESAMP = ',iesamp(i),' IBSAMP = ',ibsamp(i)
            write(io6,*)'MESHDN = ',meshdn(i)
            write(io6,*)'NGRIDY = ',ngridy,' MXNYG = ',mxnyg
            write(io6,*)'JESAMP = ',jesamp(i),' JBSAMP = ',jbsamp(i)
            stop 'Halted in RDWR1 --- See List File'
         endif
c
c ---    Check number of discrete receptors
         if(nrec(i).gt.mxrec)then
            write(io6,*)
            write(io6,*)
            write(io6,*)
            write(io6,*)'ERROR -- Too many discrete receptors for ',
     1         'array dimensions -- File: ',i
            write(io6,*)'NREC(i) = ',nrec(i),' MXREC = ',mxrec
            stop 'Halted in RDWR1 --- See List File'
         endif 
c
c ---    Check number of complex terrain receptors
         if(nctrec(i).gt.mxrect)then
            write(io6,*)
            write(io6,*)
            write(io6,*)
            write(io6,*)'ERROR -- Too many CTSG receptors for array ',
     1         'dimensions -- File: ',i
            write(io6,*)'NCTREC(i) = ',nctrec(i),' MXRECT = ',mxrect
            stop 'Halted in RDWR1 --- See List File'
         endif                                   
c
c ---    Check total number of sources
         if(ntsrc.gt.mxsrc .AND. isrcinfo(i).NE.0)then
            write(io6,*)
            write(io6,*)
            write(io6,*)
            write(io6,*)'ERROR -- Too many sources for array ',
     1         'dimensions -- File: ',i
            write(io6,*)'NTSRC = ',ntsrc,' MXSRC = ',mxsrc
            stop 'Halted in RDWR1 --- See List File'
         endif 
c
c ---    Number of surface stations should match in all files
         if(i.eq.1)then
c ---       Set output variable NSSTAI
            nsstai=nssta(i)
            if(nsstai.gt.mxssta)then
               write(io6,*)
               write(io6,*)
               write(io6,*)
               write(io6,*)'ERROR -- NSSTAI too large for array ',
     1         'dimensions'
               write(io6,*)'NSSTAI = ',nsstai,' MXSSTA = ',mxssta
               stop 'Halted in RDWR1 --- See List File'
            endif
         else
            if(nsstai.ne.nssta(i))then
               write(io6,*)
               write(io6,*)
               write(io6,*)
               write(io6,*)'ERROR in RDWR1 -- NSSTA does not ',
     1         'match expected value'
               write(io6,*)'NSSTAI = ',nsstai,' I = ',i
               write(io6,*)'NSSTA  = ',(nssta(ii),ii=1,i)
               stop 'Halted in RDWR1 --- See List File'
            endif
         endif
c
c ---    HEADER RECORD NCOM+4 -- Run title
         read(io)(title(n,i),n=1,3)
c
c ---    HEADER RECORD NCOM+5 -- List of species output
         nspeci=nspout(i)
         read(io)(csout(n,i),n=1,nspeci)

c ---    Read units for each specie/level in list (record NCOM+5a)
c ---    (introduced in Dataset 2.2)
         if(LSPUNIT) then
            read(io) (aspunit(n,i),n=1,nspeci)
         else
c ---       Older CALPUFF always has mass units
            do n=1,nspeci
               if(csout(n,i)(15:15).EQ.'F') then
                  aspunit(n,i)='g/m2/s          '
               else
                  aspunit(n,i)='g/m3            '
               endif
            enddo
         endif
c
c ---    HEADER RECORD NCOM+6 -- Discrete (non-gridded) receptor data
         nreci=nrec(i)
         if(nreci.gt.0)then
           read(io)(xrec(n,i),n=1,nreci),(yrec(n2,i),n2=1,nreci),
     1       (zrec(n3,i),n3=1,nreci)
         endif
c
c ---    HEADER RECORD NCOM+7 -- Complex terrain receptor data
         nctreci=nctrec(i)
         if(nctreci.gt.0)then
            read(io)(xrct(n,i),n=1,nctreci),(yrct(n2,i),n2=1,nctreci),
     1          (zrct(n3,i),n3=1,nctreci),(ihill(n4,i),n4=1,nctreci)
         endif

c ---   Dataset version 2.1 introduces source names
c ---   HEADER RECORDs #NCOM+8 to 15 -- Source names
        if(isrcinfo(i).EQ.1) then
           lfatal=.FALSE.
c ---      Loop over source types (points,areas,lines,volumes)
           do k=1,8
              if(nstype(i,k).GT.0 .AND. .NOT.LFATAL) then
                 n1=nstot(k)+1
                 nstot(k)=nstot(k)+nstype(i,k)
                 read(io) ktype,(csource(n,k),n=n1,nstot(k))
                 if(ktype.NE.k) lfatal=.TRUE.
              endif
           enddo
           if(LFATAL) then
c ---         Report problem information and quit
              write(*,*)
              write(*,*)
              write(*,100)'  *  Unexpected source type found, or     *'
              write(*,100)'  *  Unexpected source number found.      *'
              write(*,100)'  *  Number of sources expected:          *'
              write(*,101)'  *     NPTS',nstot(1),nstot(2)
              write(*,101)'  *   NAREAS',nstot(3),nstot(4)
              write(*,101)'  *   NLINES',nstot(5),nstot(6)
              write(*,101)'  *    NVOLS',nstot(7),nstot(8)
              write(*,100)'  *  Source Names Read:                   *'
              do k=1,8
                 do n=1,nstot(k)
                    write(*,*)cstype(k),csource(n,k)
                 enddo
              enddo
              stop
           endif
        else
c ---      Update total number of sources
           ntpts=ntpts+npts(i)
           ntareas=ntareas+nareas(i)
           ntlines=ntlines+nlines(i)
           ntvols=ntvols+nvols(i)
        endif

100     format(1x,a48)
101     format(1x,a12,2(8x,i8),'   *')

      enddo

c
c --- Define additional output variables
      lsamp=lsgrid(1)
c
c --- Sampling grid dimensions
      if(lsamp)then
         nxsam=meshdn(1)*(iesamp(1)-ibsamp(1))+1
         nysam=meshdn(1)*(jesamp(1)-jbsamp(1))+1
      else
         nxsam=0
         nysam=0
      endif
c
c ----------------------------------------------------------------------
c --- Check & write header record data to the list file
c ----------------------------------------------------------------------
c
      do j=1,nfiles
         write(io6,*)
         write(io6,*)'Title lines from file No.: ',j
         do i=1,3
            write(io6,'(5x,a80)')title(i,j)
         enddo
      enddo
c
      write(io6,*)
      write(io6,*)'Data from header records of File #1'
      write(io6,*)'   CMODEL: ',cmodel(1)
      write(io6,*)'   VER:     ',ver(1)
      write(io6,*)'   LEVEL:   ',level(1)
      write(io6,*)'   IBYR:    ',ibyr(1)
      write(io6,*)'   IBJUL:   ',ibjul(1)
      write(io6,*)'   IBHR:    ',ibhr(1)
      if(iptime(1).EQ.2) then
         write(io6,*)'   IBSEC:   ',ibsec(1)
         write(io6,*)'   NSECDT:  ',nsecdt(1)
         write(io6,*)'   XBTZ:    ',xbtz(1)
      endif
      write(io6,*)'   IRLG:    ',irlg(1)
      write(io6,*)'   IAVG:    ',iavg(1)
      write(io6,*)'   PMAP:    ',pmap(1)
      write(io6,*)'   DATUM:   ',datum(1)
      write(io6,*)'   NX:      ',nx(1)
      write(io6,*)'   NY:      ',ny(1)
      write(io6,*)'   DXKM:    ',dxkm(1)
      write(io6,*)'   DYKM:    ',dykm(1)
      write(io6,*)'   IONE:    ',ione(1)
      write(io6,*)'   XORIGKM: ',xorigkm(1)
      write(io6,*)'   YORIGKM: ',yorigkm(1)
      write(io6,*)'   NSSTA:   ',nssta(1)
      write(io6,*)'   IBCOMP:  ',ibcomp(1)
      write(io6,*)'   IECOMP:  ',iecomp(1)
      write(io6,*)'   JBCOMP:  ',jbcomp(1)
      write(io6,*)'   JECOMP:  ',jecomp(1)
      write(io6,*)'   IBSAMP:  ',ibsamp(1)
      write(io6,*)'   IESAMP:  ',iesamp(1)
      write(io6,*)'   JBSAMP:  ',jbsamp(1)
      write(io6,*)'   JESAMP:  ',jesamp(1)
      write(io6,*)'   MESHDN:  ',meshdn(1)
      write(io6,*)'   NREC:    ',nrec(1)
      write(io6,*)'   NCTREC:  ',nctrec(1)
      write(io6,*)'   LSGRID:  ',lsgrid(1)
      write(io6,*)'   NSPOUT:  ',nspout(1)
      if(isrcinfo(1).EQ.1) then
         write(io6,*)'   MSOURCE: ',msource(1)
      endif
c
      write(io6,1010)
1010  format(/1x,'File',6x,'NPTS',5x,'NAREAS',5x,'NLINES',5x,'NVOLS')
      do i=1,nfiles
      write(io6,1012)i,npts(i),nareas(i),nlines(i),nvols(i)
1012     format(1x,i4,5x,i5,6x,i5,6x,i5,5x,i5)
      enddo
c
      nspeci=nspout(1)
      write(io6,1014)(csout(n,1),n=1,nspeci)
1014  format(/1x,'SPECIES: ',20(a15,2x))
      write(io6,1015)(aspunit(n,1),n=1,nspeci)
1015  format(/1x,'  UNITS: ',20(a16,1x))
c
c --- Perform QA checks

      ierr=0
      do i=2,nfiles
         if(isrcinfo(i).ne.isrcinfo(1))ierr=ierr+1
         if(isrcindv(i).ne.isrcindv(1))ierr=ierr+1
         if(msource(i).ne.msource(1))ierr=ierr+1
         if(iptime(i).ne.iptime(1))ierr=ierr+1

c --- v1.5.0 (121203)
c --- Remove tests for matching time periods
         if(iptime(1).EQ.2) then
            if(nsecdt(i).ne.nsecdt(1))ierr=ierr+1
            if(xbtz(i).ne.xbtz(1))ierr=ierr+1
         endif

         if(nrec(i).ne.nrec(1))ierr=ierr+1
         if(nctrec(i).ne.nctrec(1))ierr=ierr+1
         if(nspout(i).ne.nspout(1))ierr=ierr+1
         if(lsgrid(i).and..not.lsgrid(1))ierr=ierr+1
         if(.not.lsgrid(i).and.lsgrid(1))ierr=ierr+1
         if(xorigkm(i)-xorigkm(1).gt.0.001)ierr=ierr+1
         if(yorigkm(i)-yorigkm(1).gt.0.001)ierr=ierr+1
c ---    Check all species names and units
         do n=1,nspout(1)
            if(csout(n,i).NE.csout(n,1))ierr=ierr+1
            if(aspunit(n,i).NE.aspunit(n,1))ierr=ierr+1
         enddo
      enddo
      if(ierr.gt.0)then
         write(io6,*)
         write(io6,*)
         write(io6,*)
         write(io6,*)'ERROR -- Header record data do not match'
         write(io6,*)'IERR    = ',ierr
         write(io6,*)
         write(io6,*)
         write(io6,*)'ISRCINFO= ',(isrcinfo(n),n=1,nfiles)
         write(io6,*)'ISRCINDV= ',(isrcindv(n),n=1,nfiles)
         write(io6,*)'MSOURCE = ',(msource(n),n=1,nfiles)
         write(io6,*)'IPTIME  = ',(iptime(n),n=1,nfiles)
         write(io6,*)'IBYR    = ',(ibyr(n),n=1,nfiles)
         write(io6,*)'IBJUL   = ',(ibjul(n),n=1,nfiles)
         write(io6,*)'IBHR    = ',(ibhr(n),n=1,nfiles)
         if(iptime(1).EQ.2) then
            write(io6,*)'IBSEC   = ',(ibsec(n),n=1,nfiles)
            write(io6,*)'NSECDT  = ',(nsecdt(n),n=1,nfiles)
            write(io6,*)'XBTZ    = ',(xbtz(n),n=1,nfiles)
         endif
         write(io6,*)'IRLG    = ',(irlg(n),n=1,nfiles)
         write(io6,*)'NREC    = ',(nrec(n),n=1,nfiles)
         write(io6,*)'NCTREC  = ',(nctrec(n),n=1,nfiles)
         write(io6,*)'NSPOUT  = ',(nspout(n),n=1,nfiles)
         write(io6,*)'LSGRID  = ',(lsgrid(n),n=1,nfiles)
         write(io6,*)'XORIGKM = ',(xorigkm(n),n=1,nfiles)
         write(io6,*)'YORIGKM = ',(yorigkm(n),n=1,nfiles)
c ---    Species names and units
         write(io6,*)'Species names and units for each file:'
         do n=1,nspout(1)
            write(io6,*)(csout(n,i),i=1,nfiles)
            write(io6,*)(aspunit(n,i),i=1,nfiles)
         enddo
         stop 'Halted in RDWR1 --- See List File'
      endif
c
c --- Determine if this is a flux file (so there will be no values
c --- at CTSG receptors)
      iflux=0
      if(csout(1,1)(13:15).eq.' DF'.or.csout(1,1)(13:15).eq.' WF')then
      iflux=1
      endif
      if(csout(1,1)(1:8).eq.' REL HUM')then
      write(io6,*)'ERROR in SUBR. RDWR1 -- Invalid file -- CSOUT = ',
     1   csout(1,1)
      write(io6,*)'CALSUM does not operate on relative humidity file'
      stop
      endif

c --- v1.5.0 (121203)
c --- Construct the time period for the output file and the start and
c --- end step for each file
      call TIMEHEAD(iptime(1),nfiles,
     &              ibyr,ibjul,ibhr,ibsec,irlg,nsecdt,
     &              kbyr,kbjul,kbhr,kbsec,krlg,nstep)
c --- Do not allow variable time-periods with files that contain
c --- individual source contributions until that pathway is checked
      if(isrcindv(1).NE.0) then
         itdiff=0
         do n=2,nfiles
            if(nstep(1,1).NE.nstep(1,n)) itdiff=1
            if(nstep(2,1).NE.nstep(2,n)) itdiff=1
         enddo
         if(itdiff.NE.0) then
            write(io6,*)'Individual source contributions in CALPUFF'
            write(io6,*)'files with different start/end times are'
            write(io6,*)'currently not processed!'
            stop 'Halted in RDWR1 --- See List File'
         endif
      endif

c
c ----------------------------------------------------------------------
c --- OUTPUT of header records for summed file
c ----------------------------------------------------------------------
c
      i2drh=i2dmet(1)
c
c --- Record #1 - File Declaration -- 24 words
      write(io10) dataset0,dataver,datamod
c --- Record #2 - Number of comment lines -- 1 word
      ncom=ncommout+1
      write(io10) ncom
c --- Record #3 to NCOM+2 (Comment record section) -- 33 words each
      write(io10) doc1
c --- Go to beginning of the scratch file with the control file images
      REWIND(iox)
c --- Loop over records
      do i=1,ncommout
         comment1=blank
         read(iox,'(a132)') comment1
         write(io10) comment1
      enddo
      CLOSE(iox)
c
c --- v1.5.0 (121203)
c --- HEADER RECORD #NCOM+3 -- General run parameters
      if(dataver.EQ.'2.0             ') then
         write(io10)cmodel(1),ver(1),level(1),kbyr,kbjul,kbhr,
     1     krlg,iavg(1),nx(1),ny(1),dxkm(1),dykm(1),ione(1),
     2     xorigkm(1),yorigkm(1),nssta(1),ibcomp(1),iecomp(1),
     3     jbcomp(1),jecomp(1),ibsamp(1),jbsamp(1),iesamp(1),
     4     jesamp(1),meshdn(1),ntpts,ntareas,ntlines,ntvols,
     5     nrec(1),nctrec(1),lsgrid(1),nspout(1),lcompro,i2dmet(1),
     6     iutmzn(1),feast(1),fnorth(1),rnlat0(1),relon0(1),
     7     xlat1(1),xlat2(1),pmap(1),utmhem(1),datum(1),daten(1),
     8     clat0(1),clon0(1),clat1(1),clat2(1)
      elseif(dataver.EQ.'2.1             ') then
         write(io10)cmodel(1),ver(1),level(1),kbyr,kbjul,kbhr,
     1     kbsec,xbtz(1),krlg,iavg(1),nsecdt(1),nx(1),ny(1),
     2     dxkm(1),dykm(1),ione(1),xorigkm(1),yorigkm(1),nssta(1),
     3     ibcomp(1),iecomp(1),jbcomp(1),jecomp(1),ibsamp(1),
     4     jbsamp(1),iesamp(1),jesamp(1),meshdn(1),nstot(1),nstot(2),
     5     nstot(3),nstot(4),nstot(5),nstot(6),nstot(7),nstot(8),
     6     msource(1),nrec(1),nctrec(1),lsgrid(1),nspout(1),lcompro,
     7     i2dmet(1),iutmzn(1),feast(1),fnorth(1),rnlat0(1),relon0(1),
     8     xlat1(1),xlat2(1),pmap(1),utmhem(1),datum(1),daten(1),
     9     clat0(1),clon0(1),clat1(1),clat2(1)
      elseif(dataver.EQ.'2.2             ') then
         write(io10)cmodel(1),ver(1),level(1),kbyr,kbjul,kbhr,
     1     kbsec,abtz(1),krlg,iavg(1),nsecdt(1),nx(1),ny(1),
     2     dxkm(1),dykm(1),ione(1),xorigkm(1),yorigkm(1),nssta(1),
     3     ibcomp(1),iecomp(1),jbcomp(1),jecomp(1),ibsamp(1),
     4     jbsamp(1),iesamp(1),jesamp(1),meshdn(1),nstot(1),nstot(2),
     5     nstot(3),nstot(4),nstot(5),nstot(6),nstot(7),nstot(8),
     6     msource(1),nrec(1),nctrec(1),lsgrid(1),nspout(1),lcompro,
     7     i2dmet(1),iutmzn(1),feast(1),fnorth(1),rnlat0(1),relon0(1),
     8     xlat1(1),xlat2(1),pmap(1),utmhem(1),datum(1),daten(1),
     9     clat0(1),clon0(1),clat1(1),clat2(1)
      endif

c
c --- HEADER RECORD #NCOM+4 -- Run title
      write(io10)(titleo(n),n=1,3)
c
c --- HEADER RECORD #NCOM+5 -- List of species output
      nspeci=nspout(1)
      write(io10)(csout(n,1),n=1,nspeci)

c --- Write units for each specie/level in list (record NCOM+5a)
c --- (introduced in Dataset 2.2)
      if(LSPUNIT) then
         write(io10) (aspunit(n,1),n=1,nspeci)
      endif
c
c --- HEADER RECORD #NCOM+6 -- Discrete (non-gridded) receptor data
      nreci=nrec(1)
      if(nreci.gt.0)then
      write(io10)(xrec(n,1),n=1,nreci),(yrec(n2,1),n2=1,nreci),
     1    (zrec(n3,1),n3=1,nreci)
      endif
c
c --- HEADER RECORD #NCOM+7 -- Complex terrain receptor data
      nctreci=nctrec(1)
      if(nctreci.gt.0)then
      write(io10)(xrct(n,1),n=1,nctreci),(yrct(n2,1),n2=1,nctreci),
     1    (zrct(n3,1),n3=1,nctreci),(ihill(n4,1),n4=1,nctreci)
      endif
c
c --- HEADER RECORDs #NCOM+8 to 15 -- Source names
      if(isrcinfo(1).EQ.1) then
c ---    Loop over source types
         do k=1,8
            if(nstot(k).GT.0) then
               write(io10) k,(csource(n,k),n=1,nstot(k))
            endif
         enddo
      endif
c
c --- Pass structure variables to arg list
      iptime_=iptime(1)
      isrcinfo_=isrcinfo(1)
      isrcindv_=isrcindv(1)
c
      if(isrcinfo(1).EQ.1) then
c ---    Echo source names to list file
         write(io6,*)
         write(io6,*)'Sources identified in all files processed'
c ---    Loop over source types
         do k=1,8
            if(nstot(k).GT.0) then
               write(io6,*)'Source type ',k
               do n=1,nstot(k)
                  write(io6,*) csource(n,k)
               enddo
            endif
         enddo
      endif

      return
      end

c----------------------------------------------------------------------
c --- v1.5.0 (121203)
      subroutine rdwrhr(nfiles,nspeci,nreci,nctreci,lsamp,nxsam,nysam,
     1 iflux,ascale,bscale,lcompr,lcompro,
     2 iptime,isrcinfo,isrcindv,ntsrc,nsrc,nstot,nuse,
     3 lend)
c----------------------------------------------------------------------
c
c --- CALSUM     Version: 1.5.0     Level: 121203             RDWRHR
c                J. Scire, EARTH TECH
c
c --- PURPOSE:  Read concentration or flux data from "nfiles" CALPUFF
c               files, sum values, and output results to a new
c               CALPUFF file
c               Scratch files are used to group source contributions
c               by source-type prior to output
c
c --- V1.5.0 (121203) from V1.32 (040726)
c                    - Allow time periods in file to be different
c
c --- V1.32 (040726) from V1.31 (040623) (DGS)
c                    - Dimension QA arrays as 1 to MXFIL+MXSRC
c                    - Update source number within each source type
c                      as records are processed from scratch files
c
c --- V1.31 (040623) from V1.3 (031017) (DGS)
c                    - Use binary OPEN for scratch files associated
c                      with DATASET 2.1 CALPUFF output files
c
c --- V1.3 (031017) from V1.2 (990210) (DGS)
c                    - CALPUFF.DAT dataset version 2.1 format
c
c --- INPUTS:
c            NFILES - integer       - Number of files to be summed
c            NSPECI - integer       - Number of species in files
c             NRECI - integer       - Number of discrete receptors
c           NCTRECI - integer       - Number of special complex terrain
c             LSAMP - logical       - Flag indicating if gridded
c                                     receptors used (T=yes, F=no)
c                                     (CTSG) receptors
c             NXSAM - integer       - No. cells in X direction of
c                                     gridded receptor grid
c             NYSAM - integer       - No. cells in Y direction of
c                                     gridded receptor grid
c             IFLUX - integer       - Flag indicating if file are
c                                     concentrations (IFLUX=0) or
c                                     wet/dry fluxes (IFLUX=1).  The
c                                     CTSG receptors have no flux data.
c ASCALE(mxfil,nspec) - real array  - Multiplicative scaling factor
c BSCALE(mxfil,nspec) - real array  - Additive scaling factor
c                                     (values scaled as A * X + B,
c                                     where A=ASCALE(i,j),
c                                           B=BSCALE(i,j), and,
c                                           I is the input file index,
c                                           J is the species index)
c     LCOMPR(mxfil) - logical array - Compression flags for each
c                                     input file (F=no, T=yes)
c           LCOMPRO - logical       - Compression flag for OUTPUT
c                                     file (F=no, T=yes)
c            IPTIME - integer       - Date-time format flag
c          ISRCINFO - integer       - Source information flag
c          ISRCINDV - integer       - Individual source contribution
c                                     flag
c             NTSRC - integer       - Total number of sources
c       NSRC(mxfil) - integer array - Number of sources in each file
c          NSTOT(8) - integer array - Number of sources of each type
c --- v1.5.0 (121203)
c       NUSE(mxfil) - integer array - Flag for which files have data
c                                     0: no data for step, do not read
c                                     1: data available
c
c       Parameters:
c             MXNXG, MXNYG, MXREC, MXRECT, MXSPEC, MXFIL, MXSRC
c             IO6, IO10, IO21, IOX0
c
c --- OUTPUT:
c              LEND - logical       - Flag indicating if end-of-file
c                                     has been reached
c
c --- RDWRHR called by: MAIN
c --- RDWRHR calls:     RDDAT, WRDAT
c----------------------------------------------------------------------
c
c --- Include parameter statements
      include 'params.sum'

      parameter(mxqa=mxfil+mxsrc)
c
      real chisam(mxnxg,mxnyg,mxspec)
      real totsam(mxnxg,mxnyg,mxspec)
      real chirec(mxrec,mxspec)
      real totrec(mxrec,mxspec)
      real chict(mxrect,mxspec)
      real totct(mxrect,mxspec)
      real tmp3(mxrec),tmp5(mxrect),tmp7(mxnxyg),tmp8(mxnxyg)
      real ascale(mxfil,mxspec),bscale(mxfil,mxspec)
      real xwork1(mxnxg,mxnyg),xwork2(mxrec),xwork3(mxrect)

      integer nyrb(mxqa),njulb(mxqa),nhrb(mxqa),nsecb(mxqa)
      integer nyre(mxqa),njule(mxqa),nhre(mxqa),nsece(mxqa)
      integer nsrc(mxfil),nstot(8)

c --- v1.5.0 (121203)
      integer nuse(mxfil)

      logical lcompr(mxfil),lcompro,lsamp,lend

      character*15 cname(mxspec)
      character*16 cnamsrc,cnamsrc0


c ---------------------------------------------------------------
c
c --- Open scratch files for storing scaled source contributions
      if(isrcindv.EQ.1) then
         do is=1,8
            io=iox0+is
            OPEN(io,form='binary',status='scratch')
         enddo
      endif
c
c --- Set QA counter
      iqa=0
c
c --- Check if active portion of the sampling arrays is less than
c --- the full dimension (if so, active portion is extracted for
c --- output purposes
      ifull=0
      if(lsamp) then
         if(nxsam.eq.mxnxg.and.nysam.eq.mxnyg)then
            ifull=1
         endif
      endif
c
c --- Initialize summed arrays
c --- Gridded receptors
      if(lsamp)then
         do 101 k=1,nspeci
         do 101 i2=1,nysam
         do 101 i1=1,nxsam
            totsam(i1,i2,k)=0.0
101      continue
      endif
c --- Discrete receptors
      if(nreci.gt.0)then
         do 103 k=1,nspeci
         do 103 i1=1,nreci
            totrec(i1,k)=0.0
103      continue
      endif
c --- CTSG receptors (concentrations only)
      if(nctreci.gt.0.and.iflux.eq.0)then
         do 105 k=1,nspeci
         do 105 i1=1,nctreci
            totct(i1,k)=0.0
105      continue
      endif

c --------------------------------------------------------------
c --- Read 1-hour of concentration/flux data from "nfiles" files
c --------------------------------------------------------------

c --- v1.5.0 (121203)
      iprt=1

      do i=1,nfiles
        io=io21+i-1

c --- v1.5.0 (121203)
c ---    Skip read/write processing of this file if no data
         if(nuse(i).EQ.0) goto 990

c ---   Loop over sources within this file
        nmax=1
        if(isrcindv.EQ.1) nmax=nsrc(i)+1
        do is=1,nmax
c
c ---     Read date/time data record
          if(iptime.EQ.1) then
            read(io,end=999)iyre,ijule,ihre
          elseif(iptime.EQ.2) then
            read(io,end=999) iyrb,ijulb,ihrb,isecb,
     &                       iyre,ijule,ihre,isece
          endif

c --- v1.5.0 (121203)
          if(iprt.eq.1.and.is.eq.1.and.ihre.eq.0) then
             print 109,iyre,ijule
             iprt=0
          endif

109       format('+','Processing YR: ',i4,' DAY: ',i3)

c ---     Read source information record
          ktype=0
          if(isrcinfo.EQ.1) then
            read(io,end=999) ktype,knum,cnamsrc,xskm,yskm
            if(ktype.EQ.0) then
c ---         Save for output
              ktype0=ktype
              knum0=knum
              cnamsrc0=cnamsrc
              xskm0=xskm
              yskm0=yskm
            endif
          endif
c
c ---     Select scratch file for this source-type
          ios=iox0+ktype
c
c ---     Assign date-time to QA arrays
          iqa=iqa+1
          nyre(iqa)=iyre
          njule(iqa)=ijule
          nhre(iqa)=ihre
          if(iptime.EQ.2) then
            nsece(iqa)=isece
            nyrb(iqa)=iyrb
            njulb(iqa)=ijulb
            nhrb(iqa)=ihrb
            nsecb(iqa)=isecb
          endif
c
          if(ktype.GT.0) then
c ---       Write date/time record to scratch file
            if(iptime.EQ.1) then
              write(ios) iyre,ijule,ihre
            elseif(iptime.EQ.2) then
              write(ios) iyrb,ijulb,ihrb,isecb,
     &                   iyre,ijule,ihre,isece
            endif
c ---       Write source information record to scratch file
            if(isrcinfo.EQ.1) then
              write(ios) ktype,knum,cnamsrc,xskm,yskm
            endif
          endif
          
c ---     Loop over species
          do k=1,nspeci
c
c ---       Assign scale factors for file (i) and species (k)
            a=ascale(i,k)
            b=bscale(i,k)
c
c ---       Gridded receptor concentrations
            if(lsamp)then
              if (lcompr(i))then
                nwords=nxsam*nysam
                read(io)ii
                call uncomprs(xwork1,ii,io,nwords,cname(k),tmp7)
              else
                call rddat(io,nxsam,nysam,cname(k),tmp7)
              endif
c
c ---         Transfer data from work array into main array and scale
c ---         (and sum if total)
              ii=0
              do i2=1,nysam
                do i1=1,nxsam
                  ii=ii+1
                  if(ktype.EQ.0) then
                    totsam(i1,i2,k)=totsam(i1,i2,k)+a*tmp7(ii)+b
                  else
                    chisam(i1,i2,k)=a*tmp7(ii)+b
                  endif
                enddo
              enddo
c
              if(ktype.GT.0) then
c ---            Write scaled data to scratch file for this source-type
                 if(ifull.eq.1)then
                   if(lcompro)then
c ---                Write a compressed data record
                     call comprs(chisam(1,1,k),mxnxyg,tmp8,mxnxyg,
     1                           cname(k),ios)
                   else
c ---                Write an uncompressed data record
                     call wrdat(ios,cname(k),chisam(1,1,k),nxsam,nysam)
                   endif
                 else
                   call xtract(chisam(1,1,k),mxnxg,mxnyg,nxsam,nysam,
     1                         tmp7)
                   if(lcompro)then
c ---                Write a compressed data record
                     nwords=nxsam*nysam
                     call comprs(tmp7,nwords,tmp8,mxnxyg,cname(k),ios)
                   else
c ---                Write an uncompressed data record
                     call wrdat(ios,cname(k),tmp7,nxsam,nysam)
                   endif
                 endif
              endif
            endif
c
c ---       Discrete receptor concentrations
            if(nreci.GT.0) then
              if(lcompr(i)) then
                read(io)ii
                nwords=nreci
                call uncomprs(xwork2,ii,io,nwords,cname(k),
     :                              chirec(1,k))
              else 
                call rddat(io,nreci,1,cname(k),chirec(1,k))
              endif
c
c ---         Scale (and sum if total)
              do ir=1,nreci
                 if(ktype.EQ.0) then
                   totrec(ir,k)=totrec(ir,k)+a*chirec(ir,k)+b
                 else
                   chirec(ir,k)=a*chirec(ir,k)+b
                 endif
              enddo
c
              if(ktype.GT.0) then
c ---            Write scaled data to scratch file for this source-type
                 if(lcompro) then
c ---              Compressed data record
                   call comprs(chirec(1,k),nreci,tmp3,mxrec,cname(k),
     1                         ios)
                 else
c ---              Uncompressed data record
                   call wrdat(ios,cname(k),chirec(1,k),nreci,1)
                 endif
              endif
            endif
c
c ---       Discrete CTSG receptor concentrations
            if(nctreci.GT.0.0.and.iflux.eq.0)then
              if (lcompr(i)) then
                read(io)ii
                nwords=nctreci
              call uncomprs(xwork3,ii,io,nwords,cname(k),
     :                      chict(1,k))
              else
                call rddat(io,nctreci,1,cname(k),chict(1,k))
              endif
c
c ---         Scale (and sum if total)
              do ir=1,nctreci
                 if(ktype.EQ.0) then
                   totct(ir,k)=totct(ir,k)+a*chict(ir,k)+b
                 else
                    chict(ir,k)=a*chict(ir,k)+b
                 endif
              enddo
c
              if(ktype.GT.0) then
c ---            Write scaled data to scratch file for this source-type
                 if(lcompro)then
c ---              Compressed data record
                   call comprs(chict(1,k),nctreci,tmp5,mxrect,cname(k),
     1                         ios)
                 else
c ---              Uncompressed data record
                   call wrdat(ios,cname(k),chict(1,k),nctreci,1)
                 endif
              endif
            endif
c
c ---     End species loop (k)
          enddo
c
c ---   End source loop (is)
        enddo

c --- v1.5.0 (121203)
c ---   Next file
990     continue
c
c --- End file loop (i)
      enddo
c
c --- Perform QA checks
      ierr=0
      do i=2,iqa
         if(nyre(i).ne.nyre(1))ierr=ierr+1
         if(njule(i).ne.njule(1))ierr=ierr+1
         if(nhre(i).ne.nhre(1))ierr=ierr+1
      enddo
      if(ierr.gt.0)then
         write(io6,*)'ERROR in SUBR. RDWRHR -- Dates do not match'
         write(io6,*)'IERR  = ',ierr
         write(io6,*)'NYR   = ',(nyre(n),n=1,iqa)
         write(io6,*)'NJUL  = ',(njule(n),n=1,iqa)
         write(io6,*)'NHR   = ',(nhre(n),n=1,iqa)
         stop
      endif
c
c ----------------------------------------------------------------------
c --- Write the updated arrays to the output file
c ----------------------------------------------------------------------
c
c --- Individual source contributions output
c ------------------------------------------
      if(isrcindv.EQ.1) then
c ---   Loop over source-type scratch files
        do i=1,8
          ios=iox0+i
          REWIND(ios)

c ---     Loop over sources
          if(nstot(i).GT.0) then
            do is=1,nstot(i)
c
c ---         Date/time data record
              if(iptime.EQ.1) then
                read(ios,end=999) iyre,ijule,ihre
                write(io10) iyre,ijule,ihre
              elseif(iptime.EQ.2) then
                read(ios,end=999)iyrb,ijulb,ihrb,isecb,
     1                           iyre,ijule,ihre,isece
                write(io10) iyrb,ijulb,ihrb,isecb,
     1                      iyre,ijule,ihre,isece
              endif
c
c ---         Source information record
              read(ios,end=999) ktype,knum,cnamsrc,xskm,yskm
c
c ---         Update source number here to current value (is)
              write(io10) ktype,is,cnamsrc,xskm,yskm
c
c ---         Loop over species
              do k=1,nspeci
c
c ---           Gridded receptors
                if(lsamp) then
                  if(lcompro)then
c ---               Compressed data record
                    read(ios)ii
                    write(io10)ii
                    call rwcomprs(ios,io10,ii,tmp7)
                  else
c ---               Uncompressed data record
                    call rwdat(ios,io10,nxsam,nysam,cname,tmp7)
                  endif
                endif
c
c ---           Discrete receptors
                if(nreci.GT.0) then
                  if(lcompro)then
c ---               Compressed data record
                    read(ios)ii
                    write(io10)ii
                    call rwcomprs(ios,io10,ii,tmp3)
                  else
c ---               Uncompressed data record
                    call rwdat(ios,io10,nreci,1,cname,tmp3)
                  endif
                endif
c
c ---           CTSG complex terrain receptors (concentrations only)
                if(nctreci.GT.0.0.and.iflux.eq.0) then
                  if(lcompro)then
c ---               Compressed data record
                    read(ios)ii
                    write(io10)ii
                    call rwcomprs(ios,io10,ii,tmp5)
                  else
c ---               Uncompressed data record
                    call rwdat(ios,io10,nctreci,1,cname,tmp5)
                  endif
                endif

c ---         End loop over species (k)
              enddo

c ---       End loop over sources (is)
            enddo

          endif

c ---   End loop over scratch files (i)
        enddo

      endif
c
c --- TOTALS output
c -----------------
c --- Write date/time data record
      if(iptime.EQ.1) then
        write(io10) nyre(1),njule(1),nhre(1)
      elseif(iptime.EQ.2) then
        write(io10) nyrb(1),njulb(1),nhrb(1),nsecb(1),
     1              nyre(1),njule(1),nhre(1),nsece(1)
      endif

      if(isrcinfo.EQ.1) then
c ---    Write source information record
         write(io10) ktype0,knum0,cnamsrc0,xskm0,yskm0
      endif
c
c --- Loop over species
      do k=1,nspeci
c
c ---    Gridded receptors
         if(lsamp) then
           if(ifull.eq.1)then
             if(lcompro)then
c ---          Write a compressed data record
               call comprs(totsam(1,1,k),mxnxyg,tmp8,mxnxyg,cname(k),
     1                     io10)
             else
c ---          Write an uncompressed data record
               call wrdat(io10,cname(k),totsam(1,1,k),nxsam,nysam)
             endif
           else
             call xtract(totsam(1,1,k),mxnxg,mxnyg,nxsam,nysam,tmp7)
             if(lcompro)then
c ---          Write a compressed data record
               nwords=nxsam*nysam
               call comprs(tmp7,nwords,tmp8,mxnxyg,cname(k),io10)
             else
c ---          Write an uncompressed data record
               call wrdat(io10,cname(k),tmp7,nxsam,nysam)
             endif
           endif
         endif
c
c ---    Discrete receptors
         if(nreci.GT.0) then
           if(lcompro)then
c ---        Compressed data record
             call comprs(totrec(1,k),nreci,tmp3,mxrec,cname(k),io10)
           else
c ---        Uncompressed data record
             call wrdat(io10,cname(k),totrec(1,k),nreci,1)
           endif
         endif
c
c ---    CTSG complex terrain receptors (concentrations only)
         if(nctreci.GT.0.0.and.iflux.eq.0) then
           if(lcompro)then
c ---        Compressed data record
             call comprs(totct(1,k),nctreci,tmp5,mxrect,cname(k),io10)
           else
c ---        Uncompressed data record
             call wrdat(io10,cname(k),totct(1,k),nctreci,1)
           endif
         endif
c
      enddo
c
c --- Close scratch files for storing scaled source contributions
      if(isrcindv.EQ.1) then
         do is=1,8
            ios=iox0+is
            CLOSE(ios)
         enddo
      endif

      lend=.false.
      return
c
c --- End-of-file reached
999   continue
      lend=.true.
      return
      end
c----------------------------------------------------------------------
      subroutine rddat(iounit,nx,ny,cname,outarr)
c----------------------------------------------------------------------
c
c --- CALSUM     Version: 1.5.0     Level: 950715              RDDAT
c                J. Scire, EARTH TECH
c
c --- PURPOSE:  Read a CALPUFF concentration or dry/wet flux
c               data record
c               (one 15-character identifier and a 2-D data array)
c
c --- INPUTS:
c          IOUNIT - integer      - Fortran unit no. of input file
c              NX - integer      - Number of grid points in the
c                                  X direction
c              NY - integer      - Number of grid points in the
c                                  Y direction
c
c --- OUTPUT:
c           CNAME - character*15 - Species identifier
c   OUTARR(nx,ny) - real array   - Array of concentration data (g/m**3)
c                                  or dry/wet flux data (g/m**2/s)
c
c --- RDDAT called by:  RDWRHR
c --- RDDAT calls:      none
c
c----------------------------------------------------------------------
c
      real outarr(nx,ny)
      character*15 cname
c
        read(iounit)cname,outarr
c
      return
      end
c----------------------------------------------------------------------
      subroutine wrdat(iounit,cname,outarr,nx,ny)
c----------------------------------------------------------------------
c
c --- CALSUM     Version: 1.5.0     Level: 950715              WRDAT
c                J. Scire, EARTH TECH
c
c --- PURPOSE:  Write a CALPUFF concentration or dry/wet flux
c               data record
c               (one 15-character identifier and a 2-D data array)
c
c --- INPUTS:
c          IOUNIT - integer      - Fortran unit no. of output file
c           CNAME - character*15 - Species identifier
c   OUTARR(nx,ny) - real array   - Array of concentration data (g/m**3)
c                                  or dry/wet flux data (g/m**2/s)
c              NX - integer      - Number of sampling grid points in the
c                                  X direction
c              NY - integer      - Number of sampling grid points in the
c                                  Y direction
c
c --- OUTPUT:  none
c
c --- WRDAT called by:  RDWRHR
c --- WRDAT calls:      none
c
c----------------------------------------------------------------------
c
      real outarr(nx,ny)
      character*15 cname
c
      write(iounit)cname,outarr
c
      return
      end
c----------------------------------------------------------------------
      subroutine rwdat(inunit,iounit,nx,ny,cname,outarr)
c----------------------------------------------------------------------
c
c --- CALSUM     Version: 1.5.0     Level: 031017              RWDAT
c                J. Scire, EARTH TECH
c
c --- PURPOSE:  Read and write a CALPUFF concentration or dry/wet flux
c               data record
c               (one 15-character identifier and a 2-D data array)
c
c --- INPUTS:
c          INUNIT - integer      - Fortran unit no. of input file
c          IOUNIT - integer      - Fortran unit no. of output file
c              NX - integer      - Number of sampling grid points in the
c                                  X direction
c              NY - integer      - Number of sampling grid points in the
c                                  Y direction
c           CNAME - character*15 - Species identifier
c   OUTARR(nx,ny) - real array   - Array of concentration data (g/m**3)
c                                  or dry/wet flux data (g/m**2/s)
c
c --- OUTPUT:  none
c
c --- RWDAT called by:  RDWRHR
c --- RWDAT calls:      none
c
c----------------------------------------------------------------------

      real outarr(nx,ny)
      character*15 cname
c
      read(inunit)cname,outarr
      write(iounit)cname,outarr
c
      return
      end
c----------------------------------------------------------------------
      subroutine rwcomprs(inunit,iounit,ii,xwork)
c----------------------------------------------------------------------
c
c --- CALSUM     Version: 1.5.0     Level: 031017           RWCOMPRS
c                J. Scire, EARTH TECH
c
c --- PURPOSE:  Read and write a compressed data record
c
c --- INPUTS:
c              INUNIT - integer    - Unit number of input file
c              IOUNIT - integer    - Unit number of output file
c                  II - integer    - Number of words in compressed
c                                    data record
c        XWORK(nwork) - real array - Compressed data array
c
c --- OUTPUT:  none
c
c --- RWCOMPRS called by: RDWRHR
c --- RWCOMPRS calls:     none
c----------------------------------------------------------------------

      real xwork(ii)
      character*15 clabel
c
      read(inunit)clabel,xwork
      write(iounit)clabel,xwork

      return
      end
c----------------------------------------------------------------------
      subroutine xtract(datarr,nxmax,nymax,nxact,nyact,outarr)
c----------------------------------------------------------------------
c
c --- CALSUM     Version: 1.5.0     Level: 950715             XTRACT
c                J. Scire, EARTH TECH
c
c --- PURPOSE:  Extract the active portion of a 2-D array
c
c --- INPUTS:
c   DATARR(nxmax,nymax) - real    - Full data array
c                 NXMAX - integer - First dimension of data array
c                 NYMAX - integer - Second dimension of data array
c                 NXACT - integer - Number of active elements of the
c                                   array (first dimension)
c                 NYACT - integer - Number of active elements of the
c                                   array (second dimension)
c        Parameters:
c           IO6
c
c --- OUTPUT:
c   OUTARR(nxact,nyact) - real    - Output array consisting only
c                                   of the active elements of the
c                                   full input array
c
c --- XTRACT called by:  RDWRHR
c --- XTRACT calls:      none
c
c----------------------------------------------------------------------
c
      include 'params.sum'
c
      real datarr(nxmax,nymax),outarr(nxact,nyact)
c
c --- Check that values of array dimensions are reasonable
      if(nxact.le.0.or.nxact.gt.nxmax.or.
     1   nyact.le.0.or.nyact.gt.nxmax)then
         write(io6,*)'ERROR in subr. XTRACT -- Invalid values ',
     1      'of array dimensions input -- NXACT = ',nxact,' NYACT = ',
     2      nyact,' NXMAX = ',nxmax,' NYMAX = ',nymax
         stop
      endif
c
c --- Extract the active portion of the input data array
      do 100 i=1,nxact
      do 100 j=1,nyact
      outarr(i,j)=datarr(i,j)
100   continue
c
      return
      end
c----------------------------------------------------------------------
      subroutine uncomprs(xwork,ii,io,nwords,clabel,xdat)
c----------------------------------------------------------------------
c
c --- CALSUM     Version: 1.5.0     Level: 960422           UNCOMPRS
c                J. Scire, EARTH TECH
c
c --- PURPOSE:  Read a compressed data records and uncompress
c               the data
c
c --- INPUTS:
c        XWORK(nwork) - real array - Work array to temporarily store
c                                    compressed array
c                  II - integer    - Number of words in compressed
c                                    data record
c                  IO - integer    - Unit number of input file
c              NWORDS - integer    - Number of values in data array
c                                    after uncompression
c      PARAMETERS: IO6
c
c --- OUTPUTS:
c              CLABEL - char*15    - Character record header
c        XDAT(nwords) - real array - Array of uncompressed data to be
c                                    output
c
c --- UNCOMPRS called by: MAIN
c --- UNCOMPRS calls:     none
c----------------------------------------------------------------------
c
c --- Include parameter statements
       include 'params.sum'
c
      real xwork(ii),xdat(nwords)
      character*15 clabel
c
c --- Read the compressed data record
      read(io)clabel,xwork
c
c --- Uncompress the data
      jj=0
      do 100 i=1,ii
      if(xwork(i).gt.0.0)then
      jj=jj+1
      xdat(jj)=xwork(i)
      else
      nzero=-xwork(i)
      do j=1,nzero
         jj=jj+1
         xdat(jj)=0.0
      enddo
      endif
100   continue
c
c --- QA check that expanded array is correct size
      if(jj.ne.nwords)then
      write(io6,*)'ERROR in Subr. UNCOMPRS -- Expanded array ',
     1   'is not the correct size -- JJ = ',jj,' NWORDS = ',nwords
      stop
      endif
c
      return
      end
c----------------------------------------------------------------------
      subroutine comprs(xdat,nwords,xwork,nwork,clabel,io)
c----------------------------------------------------------------------
c
c --- CALSUM     Version: 1.5.0     Level: 990210             COMPRS
c                J. Scire, EARTH TECH
c
c --- PURPOSE:  Compress an array of concentrations, dry fluxes,
c               or wet fluxes by replacing strings of zero values
c               with a negative code indicating the number of zero
c               values
c
c --- INPUTS:
c        XDAT(nwords) - real array - Array of uncompressed data to be
c                                    output
c              NWORDS - integer    - Number of values in data array
c        XWORK(nwork) - real array - Work array to temporarily store
c                                    compressed array
c               NWORK - integer    - Dimension of work array - NWORK
c                                    must be >= NWORDS
c              CLABEL - char*15    - Character record header
c                  IO - integer    - Unit number of output file
c
c     Parameters: IO6
c
c --- OUTPUT:  none
c
c --- COMPRS called by: OUTPUT
c --- COMPRS calls:     WRDAT
c----------------------------------------------------------------------
c
c --- Include parameter statements
      include 'params.sum'
c
      real xdat(nwords),xwork(nwork)
      character*15 clabel
c
c --- Check that work array is sized large enough
      if(nwork.lt.nwords)then
         write(io6,*)'ERROR in Subr. COMPRS -- Work array ',
     1   'dimension is too small -- NWORK = ',nwork,' NWORDS = ',
     2   nwords
         stop
      endif
c
c --- Replace all zeroes with negative coded integer
      nzero=0
      ii=0
      do 100 i=1,nwords
c
         if(xdat(i).eq.0.0)then
            nzero=nzero+1
            go to 100
         else if(xdat(i).lt.0.0)then
            write(io6,*)'ERROR in Subr. COMPRS -- Negative value ',
     1      'encountered with COMPRESS option on -- I = ',i,
     2      ' XDAT(i) = ',xdat(i)
            write(io6,*)'COMPRESS option cannot be used when data ',
     1      'values are negative'
            stop
         endif
c
         if(nzero.eq.0)then
            ii=ii+1
            xwork(ii)=xdat(i)
         else
            ii=ii+1
            xwork(ii)=-(float(nzero)+0.0001)
            nzero=0
            ii=ii+1
            xwork(ii)=xdat(i)
         endif
100   continue
c
      if(nzero.gt.0)then
         ii=ii+1
         xwork(ii)=-(float(nzero)+0.0001)
      endif
c
c --- Write the data records (header, compressed data record)
      write(io)ii
      call wrdat(io,clabel,xwork,ii,1)
c
      return
      end
c----------------------------------------------------------------------
      subroutine tfercf(title,ncommout)
c----------------------------------------------------------------------
c
c --- CALSUM   Version: 1.5.0     Level: 030402               TFERCF
c ---          D. Strimaitis,    Earth Tech, Inc.
c
c --- PURPOSE: Transfer image of control file to scratch file
c
c --- INPUTS:
c          TITLE(3) - c*80          - Control File Title Lines
c
c        Parameters: iox, io5
c
c --- OUTPUT:
c          NCOMMOUT - integer       - Number of comment records
c
c --- TFERCF called by:  MAIN
c --- TFERCF calls:      none
c----------------------------------------------------------------------
c
      include 'params.sum'

      character*80 title(3)
      character*132 aline, blank
      character*33 blank33

c --- Set blanks
      data blank33/'                                 '/
      blank(1:33)=blank33
      blank(34:66)=blank33
      blank(67:99)=blank33
      blank(100:132)=blank33

c --- Reposition to start of input control file
      REWIND(io5)

c --- Initialize number of records written to scratch file
      ncommout=0

c --- Determine number of lines in the control file before the
c --- title lines
      read(io5,*) in1
      nlines=2*in1+4

c --- Reposition input control file again
      REWIND(io5)

c --- Open scratch file
      OPEN(iox,status='scratch')

c --- Write the title records first
      do i=1,3
         write(iox,'(a80)') title(i)
         ncommout=ncommout+1
      enddo

c --- Transfer other control file records
      do i=1,nlines
         aline=blank
         read(io5,'(a132)',end=999) aline
         write(iox,'(a132)') aline
         ncommout=ncommout+1
      enddo

c --- Transfer completed
999   close(io5)

      return
      end
c----------------------------------------------------------------------
      subroutine clipr(cstring70)
c----------------------------------------------------------------------
c
c --- CALSUM   Version: 1.5.0     Level: 030709                CLIPR
c ---          D. Strimaitis,    Earth Tech, Inc.
c
c --- PURPOSE: Pad all characters in string after the first group of
c              non-blank characters
c
c --- INPUTS:
c          cstring70 - c*70          - Filename string
c
c
c --- OUTPUT:
c          cstring70 - c*70          - Filename string
c
c --- CLIPR called by:  MAIN
c --- CLIPR calls:      none
c----------------------------------------------------------------------
c

      character*70 cstring70
      character*1 blank

      data blank/' '/

c --- Find first non-blank character
      i=0
1     i=i+1
      if(cstring70(i:i).EQ.blank) then
         goto 1
      endif

c --- Now find next blank
2     i=i+1
      if(cstring70(i:i).NE.blank) then
         goto 2
      endif

c --- Overwrite all remaining characters with blanks
      do j=i,70
         cstring70(j:j)=blank
      enddo

      return
      end

c----------------------------------------------------------------------
      subroutine timehead(iptime,nfiles,
     &                    ibyr,ibjul,ibhr,ibsec,irlg,nsecdt,
     &                    kbyr,kbjul,kbhr,kbsec,krlg,nstep)
c----------------------------------------------------------------------
c
c --- CALSUM     Version: 1.5.0     Level: 121203           TIMEHEAD
c                D.Strimaitis, Exponent, Inc
c
c --- PURPOSE:  Process time periods in all files to be merged to set
c               the total period for the output file and to identify
c               the timesteps contained in each file.
c               This allows CALPUFF files with different simulation
c               periods to be merged, assuming that concentrations
c               are zero for those periods not in the file.
c
c --- INPUTS:
c            IPTIME - integer       - Date-time format flag
c            NFILES - integer       - Number of files to be summed
c       IBYR(mxfil) - integer arr   - Starting YYYY each file (year)
c      IBJUL(mxfil) - integer arr   - Starting JJJ each file (Jul Day)
c       IBHR(mxfil) - integer arr   - Starting HH each file (hour)
c      IBSEC(mxfil) - integer arr   - Starting SSSS each file (second)
c       IRLG(mxfil) - integer arr   - Number of timesteps each file
c            NSECDT - integer       - Timestep (second)
c
c       Parameters:
c             MXFIL, IO6
c
c --- OUTPUT:
c              KBYR - integer arr   - Starting YYYY output (year)
c             KBJUL - integer arr   - Starting JJJ output (Jul Day)
c              KBHR - integer arr   - Starting HH output (hour)
c             KBSEC - integer arr   - Starting SSSS output (second)
c              KRLG - integer arr   - Number of timesteps output
c    NSTEP(2,mxfil) - integer arr   - Step number to the first data in
c                                     each file, and to the last data
c                                     data in each file
c
c --- TIMEHEAD called by: (main)
c --- TIMEHEAD calls:     DEDAT, DELTSEC
c----------------------------------------------------------------------
c
c --- Include parameter statements
      include 'params.sum'

      integer ibyr(mxfil),ibjul(mxfil),ibhr(mxfil),ibsec(mxfil)
      integer irlg(mxfil),nsecdt(mxfil)
      integer nstep(2,mxfil)
      integer idathr(mxfil)

c --- Form starting date-hour variables for each file
      do n=1,nfiles
         idathr(n)=ibyr(n)*100000+ibjul(n)*100+ibhr(n)
      enddo

c --- Find earliest start-time
      kdathr=idathr(1)
      do n=2,nfiles
         kdathr=MIN(kdathr,idathr(n))
      enddo
      kbsec=0
      if(iptime.EQ.2) then
c ---    Includes seconds, and time is a begin-time
         kbsec=3600
         do n=1,nfiles
            if(kdathr.EQ.idathr(n)) kbsec=MIN(kbsec,ibsec(n))
         enddo
      endif
c --- Decode start time for output
      call DEDAT(kdathr,kbyr,kbjul,kbhr)

c --- Set starting and ending timestep for each file, and
c --- total number of steps in output
      krlg=0
      do n=1,nfiles
         call DELTSEC(kdathr,kbsec,idathr(n),ibsec(n),ndelsec)
         nstep(1,n)=1+ndelsec/nsecdt(n)
         nstep(2,n)=nstep(1,n)+irlg(n)-1
         krlg=MAX(krlg,nstep(2,n))
      enddo

c --- Report results
      write(io6,*)
      write(io6,*)'TIMEHEAD Results for output file'
      write(io6,*)'    Year  Julday    Hour  Second   Steps'
      write(io6,'(5i8)') kbyr,kbjul,kbhr,kbsec,krlg
      write(io6,*)
      write(io6,*)'TIMEHEAD Results for individual files'
      write(io6,*)'    File    Year  Julday    Hour  Second   Steps'//
     &            '   First    Last'
      do n=1,nfiles
         write(io6,'(8i8)') n,ibyr(n),ibjul(n),ibhr(n),ibsec(n),
     &                      irlg(n),(nstep(k,n),k=1,2)
      enddo
      write(io6,*)

      return
      end
